@@ -1,8 +1,11 @@
+import datetime
+
 __author__ = 'tdpreece'
 import logging
 import time
 import json
 from collections import OrderedDict
+from threading import Timer
 
 import stomp
 
@@ -11,24 +14,39 @@ logger.addHandler(logging.NullHandler())
 
 
 class Client(object):
-    def __init__(self, hostname, unique_id, port=61613, time_to_wait_for_requests=1):
+    def __init__(self, hostname, unique_id, port=61613, request_timeout_millis=500):
         self.hostname = hostname
         self.port = port
         self.unique_id = unique_id
-        self.time_to_wait_for_requests = time_to_wait_for_requests
+        self.request_timeout_millis = request_timeout_millis
+        self.total_processing_time_millis = 0
+        self.__start_time = None
+        self.__end_time = None
+        self.__timer = None
+
+    def get_request_timeout_millis(self):
+        return self.request_timeout_millis
 
     def go_live_with(self, processing_rules):
+        self.__start_time = datetime.datetime.now()
         self.run(ApplyProcessingRules(processing_rules))
 
     def run(self, handling_strategy):
         try:
             print('Starting client')
-            remote_broker = RemoteBroker(self.hostname, self.port, self.unique_id)
+
+            remote_broker = RemoteBroker(self.hostname, self.port, self.unique_id, self.request_timeout_millis)
             remote_broker.subscribe(handling_strategy)
+
             print('Waiting for requests')
-            time.sleep(self.time_to_wait_for_requests)
-            print('Stopping client')
-            remote_broker.close()
+            # DEBT - this is just to block.
+            while remote_broker.is_connected():
+                time.sleep(0.1)
+
+            self.__end_time = datetime.datetime.now()
+            difference = self.__end_time - self.__start_time
+            self.total_processing_time_millis = difference.total_seconds() * 1000.00
+
         except Exception as e:
             print('There was a problem processing messages')
             logger.exception('Problem communicating with the broker.')
@@ -107,22 +125,27 @@ class ApplyProcessingRules(object):
 
 
 class Listener(stomp.ConnectionListener):
-    def __init__(self, remote_broker, handling_strategy):
+    def __init__(self, remote_broker, handling_strategy, start_timer, stop_timer):
         self.remote_broker = remote_broker
         self.handling_strategy = handling_strategy
+        self.start_timer = start_timer
+        self.stop_timer = stop_timer
 
     def on_message(self, headers, message):
+        self.stop_timer()
         self.handling_strategy.process_next_message_from(self.remote_broker, headers, message)
+        self.start_timer()
 
 
 class RemoteBroker(object):
-    def __init__(self, hostname, port, unique_id):
+    def __init__(self, hostname, port, unique_id, request_timeout_millis):
         hosts = [(hostname, port)]
         connect_timeout = 10
         self.conn = stomp.Connection(host_and_ports=hosts, timeout=connect_timeout)
         self.conn.start()
         self.conn.connect(wait=True)
         self.unique_id = unique_id
+        self.request_timeout_millis = request_timeout_millis
 
     def acknowledge(self, headers):
         self.conn.ack(headers['message-id'], headers['subscription'])
@@ -134,13 +157,26 @@ class RemoteBroker(object):
         )
 
     def subscribe(self, handling_strategy):
-        listener = Listener(self, handling_strategy)
+        listener = Listener(self, handling_strategy, self.start_timer, self.stop_timer)
         self.conn.set_listener('listener', listener)
         self.conn.subscribe(
                 destination='{}.req'.format(self.unique_id),
                 id=1,
                 ack='client-individual'
         )
+        self.start_timer()
 
     def close(self):
+        print('Stopping client')
         self.conn.disconnect()
+
+    def is_connected(self):
+        return self.conn.is_connected()
+
+    def stop_timer(self):
+        if self.__timer is not None:
+            self.__timer.cancel()
+
+    def start_timer(self):
+        self.__timer = Timer(self.request_timeout_millis / 1000.00, self.close)
+        self.__timer.start()
